;===============Very Fast Gray Sprite Routine (16x16) by Donald Straney==============
;
;335 bytes/16443 cycles without clipping, 504 bytes/17073 cycles with clipping (worst case)
;Inputs: hl-Pointer to sprite, dark layer first, then light layer,
;		 then mask
;		 bc=x,y position to draw sprite
;
;Outputs: sprite drawn to screen, b=0, c=14
;Destroyed: af,bc,de,hl,ix
;

layer1	equ	$fc00
layer2	equ	$ca00

VeryFastGraySprite16:
#ifdef VFGSPR16_CLIP
VeryFastGraySprite16_testvclip1:
	ld a,c
	sub 64-16
	jr c,VeryFastGraySprite16_novclip
	ld e,a
	ld a,16
	sub e
	ret z
	jr nc,VeryFastGraySprite16_endvclip
VeryFastGraySprite16_testvclip2:
	xor a
	sub c
	cp 16
	ret nc
	ld e,a
	add a,a
	add a,l
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	ld a,16
	sub e
	ld c,0
	jr VeryFastGraySprite16_endvclip
VeryFastGraySprite16_novclip:
	ld a,16
VeryFastGraySprite16_endvclip:
	ld (VeryFastGraySprite16_rows),a
	ld a,b
	cp 128-16
	jr c,VeryFastGraySprite16_nohclip
	cp 128-8
	jr c,VeryFastGraySprite16_testhclip4		;clip right byte only
	cp -16
	jr c,VeryFastGraySprite16_testhclip3		;clip left byte, cut right
	cp -8
	jr c,VeryFastGraySprite16_testhclip2		;clip right byte, cut left
VeryFastGraySprite16_testhclip1:				;clip left byte only
	ld a,b
	add a,128
	ld b,a
	ld ix,-16
	push bc
	ld de,$0077
	ld bc,$7777
	jr VeryFastGraySprite16_endhclip
VeryFastGraySprite16_testhclip2:
	ld a,b
	add a,128
	ld b,a
	ld ix,-16
	push bc
	ld de,$0000
	ld bc,$0077
	jr VeryFastGraySprite16_endhclip
VeryFastGraySprite16_testhclip3:
	cp 128
	ret nc
	push bc
	ld de,$7700
	ld bc,$0000
	ld ix,0
	jr VeryFastGraySprite16_endhclip
VeryFastGraySprite16_testhclip4:
	push bc
	ld de,$7777
	ld bc,$7700
	ld ix,0
	jr VeryFastGraySprite16_endhclip
VeryFastGraySprite16_nohclip:
	push bc
	ld de,$7777
	ld bc,$7777
	ld ix,0
VeryFastGraySprite16_endhclip:
	ld (VeryFastGraySprite16_vertadjust),ix
	ld a,d					;d=left byte left section
	ld (VeryFastGraySprite16_hl1),a
	ld (VeryFastGraySprite16_hl2),a
	ld a,b					;b=left byte right section
	ld (VeryFastGraySprite16_hl3),a
	ld (VeryFastGraySprite16_hl4),a
	ld a,e					;e=right byte left section
	ld (VeryFastGraySprite16_hr1),a
	ld (VeryFastGraySprite16_hr2),a
	ld a,c					;c=right byte right section
	ld (VeryFastGraySprite16_hr3),a
	ld (VeryFastGraySprite16_hr4),a
	pop bc
#endif
	push hl
	ld l,c					;find location on screen
	ld h,0					;hl=y
	add hl,hl				;*2
	add hl,hl				;*4
	add hl,hl				;*8
	add hl,hl				;*16
	ld a,b					;a=x
	and %11111000
	rra						;/2
	rra						;/4
	rra						;/8
	add a,l					;add that to offset
	ld l,a					;hl=address (offset)
	ld a,h
	adc a,0
	ld h,a
#ifdef VFGSPR16_CLIP
VeryFastGraySprite16_vertadjust	equ	$+1
	ld de,0
	add hl,de
#endif
	ld a,b
	and %00000111			;a=shift amount
VeryFastGraySprite16_params:
	ld b,a					;find amount to patch shift code
	ld a,7
	sub b					;a=(7-shift)
	add a,a
	ld c,a
	add a,a
	add a,c					;a=(7-shift)*6
	ld (VeryFastGraySprite16_shiftr),a	;jr [right=(7-shift)*6]
	ld (VeryFastGraySprite16_shiftr2),a
	ld a,b
	add a,a
	ld c,a
	add a,a
	add a,c					;a=shift*6
	ld (VeryFastGraySprite16_shiftl),a	;jr [left=shift*6]
	ld (VeryFastGraySprite16_shiftl2),a
	pop ix					;restore sprite address
	push hl					;and find addresses of layers
	ld de,layer2
	add hl,de
	ex de,hl				;de=layer2
	pop hl
	ld bc,layer1
	add hl,bc				;hl=layer1
VeryFastGraySprite16_rows	equ	$+1
	ld b,16					;init row counter
VeryFastGraySprite16_loop:
	push bc					;save counter
	ld b,(ix+0)				;and get first layer in b
	ld c,(ix+32)			;second layer in c
	ld a,(ix+64)			;mask in a
	push af					;(save them for next byte)
	push bc
VeryFastGraySprite16_shiftr	equ	$+1
	jr $
	srl a					;and shift all of them!
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	cpl
	push af
	and (hl)				;and draw to layer 1
	or b
VeryFastGraySprite16_hl1	equ	$
	ld (hl),a
	pop af
	ex de,hl				;then layer 2
	and (hl)
	or c
VeryFastGraySprite16_hl2	equ	$
	ld (hl),a
	inc hl					;move to next byte
	inc de
	pop bc					;and restore the gfx for drawing the next byte
	pop af
VeryFastGraySprite16_shiftl	equ	$+1
	jr $
	sla a					;shift all of them left this time
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	cpl
	push af
	and (hl)				;again, draw to layer 2
	or c
VeryFastGraySprite16_hr1	equ	$
	ld (hl),a
	pop af
	ex de,hl				;and layer 1
	and (hl)
	or b
VeryFastGraySprite16_hr2	equ	$
	ld (hl),a
	inc ix					;point to the next byte in sprite
	ld b,(ix+0)				;and repeat the process for the right side!
	ld c,(ix+32)
	ld a,(ix+64)
	push af
	push bc
VeryFastGraySprite16_shiftr2	equ	$+1
	jr $
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	cpl
	push af
	and (hl)
	or b
VeryFastGraySprite16_hl3	equ	$
	ld (hl),a
	pop af
	ex de,hl
	and (hl)
	or c
VeryFastGraySprite16_hl4	equ	$
	ld (hl),a
	inc hl
	inc de
	pop bc
	pop af
VeryFastGraySprite16_shiftl2	equ	$+1
	jr $
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	cpl
	push af
	and (hl)
	or c
VeryFastGraySprite16_hr3	equ	$
	ld (hl),a
	pop af
	ex de,hl
	and (hl)
	or b
VeryFastGraySprite16_hr4	equ	$
	ld (hl),a
	inc ix
	ld bc,14				;move to next line
	add hl,bc
	ex de,hl
	add hl,bc
	ex de,hl
	pop bc					;restore loop counter
	dec b
	jp nz,VeryFastGraySprite16_loop	;and loop!
	ret

.end