;===============Very Fast Gray Sprite Routine (8x8) by Donald Straney==============
;
;207 bytes/4609 cycles without clipping, 305 bytes/5011 cycles with clipping (worst case)
;Inputs: hl-Pointer to sprite, dark layer first, then light layer,
;		 then mask
;		 bc=x,y position to draw sprite
;
;Outputs: sprite drawn to screen, b=0, c=15
;Destroyed: af,bc,de,hl,ix
;

layer1	equ	$fc00
layer2	equ	$ca00

VeryFastGraySprite8:
#ifdef VFGSPR8_CLIP
VeryFastGraySprite8_testvclip1:
	ld a,c
	sub 64-8					;should it be clipped on the bottom of the screen?
	jr c,VeryFastGraySprite8_novclip
	ld e,a
	ld a,8
	sub e						;if so, see how much should be clipped
	ret z
	jr nc,VeryFastGraySprite8_endvclip	;and set the number of rows to draw from that
VeryFastGraySprite8_testvclip2:
	xor a						;if it's clipped on the top,
	sub c
	cp 8
	ret nc
	ld e,a
	add a,l						;offset the sprite pointer
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	ld a,8
	sub e
	ld c,0						;and draw that at the top of the screen
	jr VeryFastGraySprite8_endvclip
VeryFastGraySprite8_novclip:
	ld a,8
VeryFastGraySprite8_endvclip:
	ld (VeryFastGraySprite8_rows),a
	ld a,b
	cp 128-8					;should it be clipped on the right?
	jr c,VeryFastGraySprite8_nohclip
	cp -8						;should it be clipped on the left?
	jr c,VeryFastGraySprite8_testhclip2
VeryFastGraySprite8_testhclip1:
	ld d,0
	ld e,$77
	ld a,b
	add a,128
	ld b,a
	ld ix,-16
	jr VeryFastGraySprite8_endhclip
VeryFastGraySprite8_testhclip2:
	cp 128
	ret nc
	ld d,$77
	ld e,0
	ld ix,0
	jr VeryFastGraySprite8_endhclip
VeryFastGraySprite8_nohclip:
	ld d,$77
	ld e,$77
	ld ix,0
VeryFastGraySprite8_endhclip:
	ld (VeryFastGraySprite8_vertadjust),ix
	ld a,d
	ld (VeryFastGraySprite8_hl1),a
	ld (VeryFastGraySprite8_hl2),a
	ld a,e
	ld (VeryFastGraySprite8_hr1),a
	ld (VeryFastGraySprite8_hr2),a
#endif
	push hl
	ld l,c					;find location on screen
	ld h,0					;hl=y
	add hl,hl				;*2
	add hl,hl				;*4
	add hl,hl				;*8
	add hl,hl				;*16
	ld a,b					;a=x
	and %11111000
	rra						;/2
	rra						;/4
	rra						;/8
	add a,l					;add that to offset
	ld l,a					;hl=address (offset)
	ld a,h
	adc a,0
	ld h,a
#ifdef VFGSPR8_CLIP
VeryFastGraySprite8_vertadjust	equ	$+1
	ld de,0
	add hl,de
#endif
	ld a,b
	and %00000111			;a=shift amount
VeryFastGraySprite8_params:
	ld b,a					;find amount to patch shift code
	ld a,7
	sub b					;a=(7-shift)
	add a,a
	ld c,a
	add a,a
	add a,c					;a=(7-shift)*6
	ld (VeryFastGraySprite8_shiftr),a	;jr [right=(7-shift)*6]
	ld a,b
	add a,a
	ld c,a
	add a,a
	add a,c					;a=shift*6
	ld (VeryFastGraySprite8_shiftl),a	;jr [left=shift*6]
	pop ix					;restore sprite address
	push hl					;and find addresses of layers
	ld de,layer2
	add hl,de
	ex de,hl				;de=layer2
	pop hl
	ld bc,layer1
	add hl,bc				;hl=layer1
VeryFastGraySprite8_rows	equ	$+1
	ld b,8					;init row counter
VeryFastGraySprite8_loop:
	push bc					;save counter
	ld b,(ix+0)				;and get first layer in b
	ld c,(ix+8)				;second layer in c
	ld a,(ix+16)			;mask in a
	push af					;(save them for next byte)
	push bc
VeryFastGraySprite8_shiftr	equ	$+1
	jr $
	srl a					;and shift all of them!
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	cpl
	push af
	and (hl)				;and draw to layer 1
	or b
VeryFastGraySprite8_hl1	equ	$
	ld (hl),a
	pop af
	ex de,hl				;then layer 2
	and (hl)
	or c
VeryFastGraySprite8_hl2	equ	$
	ld (hl),a
	inc hl					;move to next byte
	inc de
	pop bc					;and restore the gfx for drawing the next byte
	pop af
VeryFastGraySprite8_shiftl	equ	$+1
	jr $
	sla a					;shift all of them left this time
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	cpl
	push af
	and (hl)				;again, draw to layer 2
	or c
VeryFastGraySprite8_hr1	equ	$
	ld (hl),a
	pop af
	ex de,hl				;and layer 1
	and (hl)
	or b
VeryFastGraySprite8_hr2	equ	$
	ld (hl),a
	ld bc,15				;now move to next line
	add hl,bc
	ex de,hl
	add hl,bc
	ex de,hl
	inc ix					;point to the next byte in sprite
	pop bc					;restore loop counter
	dec b
	jp nz,VeryFastGraySprite8_loop	;and loop!
	ret

.end